@@ -1,4 +1,4 @@
-# Copyright 2022 Avaiga Private Limited
+# Copyright 2023 Avaiga Private Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
@@ -11,30 +11,37 @@
 
 from .config import Config
 
+
 # Default config loaded by app.py
 default_config: Config = {
-    "port": 5000,
-    "dark_mode": True,
-    "debug": False,
-    "host": "127.0.0.1",
-    "use_reloader": False,
-    "time_zone": None,
-    "propagate": True,
-    "favicon": None,
-    "title": None,
-    "theme": None,
-    "light_theme": None,
-    "dark_theme": None,
-    "use_arrow": False,
-    "system_notification": False,
-    "notification_duration": 3000,
-    "single_client": False,
-    "ngrok_token": "",
-    "upload_folder": None,
-    "flask_log": False,
-    "margin": "1em",
-    "run_browser": True,
-    "watermark": "Taipy inside",
+    "allow_unsafe_werkzeug": False,
+    "async_mode": "gevent",
     "change_delay": None,
+    "chart_dark_template": None,
+    "dark_mode": True,
+    "dark_theme": None,
+    "debug": False,
     "extended_status": False,
+    "favicon": None,
+    "flask_log": False,
+    "host": "127.0.0.1",
+    "light_theme": None,
+    "margin": "1em",
+    "ngrok_token": "",
+    "notification_duration": 3000,
+    "propagate": True,
+    "run_browser": True,
+    "run_in_thread": False,
+    "run_server": True,
+    "single_client": False,
+    "system_notification": False,
+    "theme": None,
+    "time_zone": None,
+    "title": None,
+    "upload_folder": None,
+    "use_arrow": False,
+    "use_reloader": False,
+    "watermark": "Taipy inside",
+    "webapp_path": None,
+    "port": 5000,
 }
