@@ -1,4 +1,4 @@
-# Copyright 2022 Avaiga Private Limited
+# Copyright 2023 Avaiga Private Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
@@ -19,66 +19,83 @@ from importlib.util import find_spec
 import pytz
 import tzlocal
 from dotenv import dotenv_values
+from werkzeug.serving import is_running_from_reloader
 
+from taipy.logger._taipy_logger import _TaipyLogger
+
+from ._gui_cli import _GuiCLI
 from ._page import _Page
 from .partial import Partial
+from .utils import _is_in_notebook
 
 ConfigParameter = t.Literal[
-    "port",
-    "dark_mode",
-    "debug",
-    "host",
-    "use_reloader",
-    "time_zone",
-    "propagate",
-    "favicon",
-    "title",
-    "theme",
-    "light_theme",
-    "dark_theme",
-    "use_arrow",
-    "system_notification",
-    "notification_duration",
-    "single_client",
-    "ngrok_token",
-    "upload_folder",
-    "data_url_max_size",
-    "flask_log",
-    "margin",
-    "run_browser",
-    "watermark",
+    "allow_unsafe_werkzeug",
+    "async_mode",
     "change_delay",
+    "chart_dark_template",
+    "dark_mode",
+    "dark_theme",
+    "data_url_max_size",
+    "debug",
     "extended_status",
+    "favicon",
+    "flask_log",
+    "host",
+    "light_theme",
+    "margin",
+    "ngrok_token",
+    "notification_duration",
+    "propagate",
+    "run_browser",
+    "run_in_thread",
+    "run_server",
+    "single_client",
+    "system_notification",
+    "theme",
+    "time_zone",
+    "title",
+    "upload_folder",
+    "use_arrow",
+    "use_reloader",
+    "watermark",
+    "webapp_path",
+    "port",
 ]
 
 Config = t.TypedDict(
     "Config",
     {
-        "port": int,
+        "allow_unsafe_werkzeug": bool,
+        "async_mode": str,
+        "change_delay": t.Optional[int],
+        "chart_dark_template": t.Optional[t.Dict[str, t.Any]],
         "dark_mode": bool,
+        "dark_theme": t.Optional[t.Dict[str, t.Any]],
+        "data_url_max_size": t.Optional[int],
         "debug": bool,
-        "host": str,
-        "use_reloader": bool,
-        "time_zone": t.Union[str, None],
-        "propagate": bool,
-        "favicon": t.Union[str, None],
-        "title": t.Union[str, None],
-        "theme": t.Union[t.Dict[str, t.Any], None],
-        "light_theme": t.Union[t.Dict[str, t.Any], None],
-        "dark_theme": t.Union[t.Dict[str, t.Any], None],
-        "use_arrow": bool,
-        "system_notification": bool,
-        "notification_duration": int,
-        "single_client": bool,
-        "ngrok_token": str,
-        "upload_folder": t.Union[str, None],
-        "data_url_max_size": t.Union[int, None],
-        "flask_log": bool,
-        "margin": t.Union[str, None],
-        "run_browser": bool,
-        "watermark": t.Union[str, None],
-        "change_delay": t.Union[int, None],
         "extended_status": bool,
+        "favicon": t.Optional[str],
+        "flask_log": bool,
+        "host": str,
+        "light_theme": t.Optional[t.Dict[str, t.Any]],
+        "margin": t.Optional[str],
+        "ngrok_token": str,
+        "notification_duration": int,
+        "propagate": bool,
+        "run_browser": bool,
+        "run_in_thread": bool,
+        "run_server": bool,
+        "single_client": bool,
+        "system_notification": bool,
+        "theme": t.Optional[t.Dict[str, t.Any]],
+        "time_zone": t.Optional[str],
+        "title": t.Optional[str],
+        "upload_folder": t.Optional[str],
+        "use_arrow": bool,
+        "use_reloader": bool,
+        "watermark": t.Optional[str],
+        "webapp_path": t.Optional[str],
+        "port": int,
     },
     total=False,
 )
@@ -130,25 +147,10 @@ class _Config(object):
         # return user defined IANA Time Zone (https://en.wikipedia.org/wiki/List_of_tz_database_time_zones)
         return tz
 
-    def _init_argparse(self):
-        parser = argparse.ArgumentParser()
-        parser.add_argument("-P", "--port", nargs="?", default="", const="", help="Specify server port")
-        parser.add_argument("-H", "--host", nargs="?", default="", const="", help="Specify server host")
+    def _handle_argparse(self):
+        _GuiCLI._create_parser()
+        args = _GuiCLI._parse_arguments()
 
-        parser.add_argument("--ngrok-token", nargs="?", default="", const="", help="Specify NGROK Authtoken")
-
-        debug_group = parser.add_mutually_exclusive_group()
-        debug_group.add_argument("--debug", help="Turn on debug", action="store_true")
-        debug_group.add_argument("--no-debug", help="Turn off debug", action="store_true")
-
-        reloader_group = parser.add_mutually_exclusive_group()
-        reloader_group.add_argument("--use-reloader", help="Auto reload on code changes", action="store_true")
-        reloader_group.add_argument("--no-reloader", help="No reload on code changes", action="store_true")
-
-        args, unknown_args = parser.parse_known_args()
-        self._handle_argparse(args)
-
-    def _handle_argparse(self, args):  # pragma: no cover
         config = self.config
         if args.port:
             if not _Config.__RE_PORT_NUMBER.match(args.port):
@@ -167,6 +169,10 @@ class _Config(object):
             config["use_reloader"] = False
         if args.ngrok_token:
             config["ngrok_token"] = args.ngrok_token
+        if args.webapp_path:
+            config["webapp_path"] = args.webapp_path
+        elif os.environ.get("TAIPY_GUI_WEBAPP_PATH"):
+            config["webapp_path"] = os.environ.get("TAIPY_GUI_WEBAPP_PATH")
 
     def _build_config(self, root_dir, env_filename, kwargs):  # pragma: no cover
         config = self.config
@@ -192,8 +198,9 @@ class _Config(object):
                         warnings.warn(
                             f"Invalid env value in Gui.run: {key} - {value}. Unable to parse value to the correct type.\n{e}"
                         )
+
         # Load from system arguments
-        self._init_argparse()
+        self._handle_argparse()
 
         # Taipy-config
         if find_spec("taipy") and find_spec("taipy.config"):
@@ -205,47 +212,35 @@ class _Config(object):
             except KeyError:
                 warnings.warn("taipy-config section for taipy-gui is not initialized")
 
+    def __log_outside_reloader(self, logger, msg):
+        if not is_running_from_reloader():
+            logger.info(msg)
 
-def _register_gui_config():
-    if not find_spec("taipy") or not find_spec("taipy.config"):
-        return
-    from copy import copy
+    def resolve(self):
+        app_config = self.config
+        logger = _TaipyLogger._get_logger()
+        # Special config for notebook runtime
+        if _is_in_notebook() or app_config["run_in_thread"] and not app_config["single_client"]:
+            app_config["single_client"] = True
+            self.__log_outside_reloader(logger, "Running in 'single_client' mode in notebook environment")
 
-    from taipy.config import Config as TaipyConfig
-    from taipy.config import UniqueSection
+        if app_config["run_server"] and app_config["ngrok_token"] and app_config["use_reloader"]:
+            app_config["use_reloader"] = False
+            self.__log_outside_reloader(
+                logger, "'use_reloader' parameter will not be used when 'ngrok_token' parameter is available"
+            )
 
-    from ._default_config import default_config
+        if app_config["use_reloader"] and not app_config["debug"]:
+            app_config["debug"] = True
+            self.__log_outside_reloader(logger, "application is running in 'debug' mode")
 
-    class _GuiSection(UniqueSection):
+        if app_config["debug"] and not app_config["allow_unsafe_werkzeug"]:
+            app_config["allow_unsafe_werkzeug"] = True
+            self.__log_outside_reloader(logger, "'allow_unsafe_werkzeug' has been set to True")
 
-        name = "gui"
-
-        def __init__(self, property_list: t.Optional[t.List] = None, **properties):
-            self._property_list = property_list
-            super().__init__(**properties)
-
-        def __copy__(self):
-            return _GuiSection(property_list=copy(self._property_list), **copy(self._properties))
-
-        def _to_dict(self):
-            as_dict = {}
-            as_dict.update(self._properties)
-            return as_dict
-
-        @classmethod
-        def _from_dict(cls, as_dict: t.Dict[str, t.Any], *_):
-            return _GuiSection(property_list=list(default_config), **as_dict)
-
-        def _update(self, as_dict: t.Dict[str, t.Any]):
-            if self._property_list:
-                as_dict = {k: v for k, v in as_dict.items() if k in self._property_list}
-            self._properties.update(as_dict)
-
-        @staticmethod
-        def _configure(**properties):
-            section = _GuiSection(property_list=list(default_config), **properties)
-            TaipyConfig._register(section)
-            return TaipyConfig.unique_sections[_GuiSection.name]
-
-    TaipyConfig._register_default(_GuiSection(property_list=list(default_config)))
-    TaipyConfig.configure_gui = _GuiSection._configure
+        if app_config["debug"] and app_config["async_mode"] != "threading":
+            app_config["async_mode"] = "threading"
+            self.__log_outside_reloader(
+                logger,
+                "'async_mode' parameter has been overridden to 'threading'. Using Flask built-in development server with debug mode",
+            )
