@@ -1,4 +1,4 @@
-# Copyright 2022 Avaiga Private Limited
+# Copyright 2023 Avaiga Private Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
@@ -126,7 +126,7 @@ class Element:
         self,
         name,
         gui: "Gui",
-        properties: t.Union[t.Dict[str, t.Any], None],
+        properties: t.Optional[t.Dict[str, t.Any]],
         lib: "ElementLibrary",
         is_html: t.Optional[bool] = False,
     ) -> t.Union[t.Any, t.Tuple[str, str]]:
@@ -205,7 +205,7 @@ class ElementLibrary(ABC):
         - What is this name used for?
         - What if two libraries with the same same get added to the Gui?
         """
-        return NotImplemented
+        return NotImplementedError  # type: ignore
 
     def get_js_module_name(self) -> str:
         """
@@ -225,6 +225,8 @@ class ElementLibrary(ABC):
     def get_scripts(self) -> t.List[str]:
         """
         Returns the list of resources names for the scripts.
+        If a resource name is an absolute URL it will be used as is, if it's not it will be
+        passed to get_resource to retrieve a local Path to the resource.
 
         The default implementation returns an empty list, indicating that this library contains
         no custom visual elements.
