@@ -1,4 +1,4 @@
-# Copyright 2022 Avaiga Private Limited
+# Copyright 2023 Avaiga Private Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
@@ -24,6 +24,7 @@ import typing as t
 import warnings
 from importlib import util
 from types import FrameType
+from urllib.parse import urlparse
 
 import __main__
 import markdown as md_lib
@@ -33,6 +34,8 @@ from flask import __version__ as flask_version  # type: ignore
 from flask import g, jsonify, request, send_file, send_from_directory
 from werkzeug.utils import secure_filename
 
+from taipy.logger._taipy_logger import _TaipyLogger
+
 if util.find_spec("pyngrok"):
     from pyngrok import ngrok
 
@@ -213,7 +216,9 @@ class Gui:
                 you want to access them using only '_assets_' in your application, you can
                 set _path_mapping={"assets": "/home/me/app_assets"}_. If your application
                 then requests the file _"/assets/images/logo.png"_, the server searches
-                for the file  _"/home/me/app_assets/images/logo.png"_.
+                for the file  _"/home/me/app_assets/images/logo.png"_.<br/>
+                If empty or not defined, access through the browser to all resources under the directory of the main python file
+                is allowed.
             env_filename (Optional[str]): An optional file from which to load application
                 configuration variables (see the
                 [Configuration](../gui/configuration.md#configuring-the-gui-instance) section
@@ -731,10 +736,10 @@ class Gui:
                     payload,
                     to=self.__get_ws_receiver(),
                 )
-                time.sleep(0)
+                time.sleep(0.001)
                 if ack_id := self._get_ack_id():
                     self._server._ws.emit("message", {"type": _WsType.ACKNOWLEDGEMENT.value, "id": ack_id})
-                    time.sleep(0)
+                    time.sleep(0.001)
             except Exception as e:
                 warnings.warn(f"Exception raised in Web Socket communication in '{self.__frame.f_code.co_name}':\n{e}")
         else:
@@ -911,10 +916,14 @@ class Gui:
             args = args[:arg_count]
         return user_function(*args)
 
-    def _call_user_callback(self, state_id: t.Optional[str], user_callback: t.Callable, args: t.List[t.Any]) -> t.Any:
+    def _call_user_callback(
+        self, state_id: t.Optional[str], user_callback: t.Callable, args: t.List[t.Any], module_context: t.Optional[str]
+    ) -> t.Any:
         try:
             with self.get_flask_app().app_context():
                 self.__set_client_id_in_context(state_id)
+                if module_context is not None:
+                    self._set_locals_context(module_context)
                 return self._call_function_with_state(user_callback, args)
         except Exception as e:
             if not self._call_on_exception(user_callback.__name__, e):
@@ -1092,7 +1101,7 @@ class Gui:
         # Update variable directory
         self.__var_dir.add_frame(page._frame)
 
-    def add_pages(self, pages: t.Union[t.Dict[str, t.Union[str, Page]], str] = None) -> None:
+    def add_pages(self, pages: t.Optional[t.Union[t.Dict[str, t.Union[str, Page]], str]] = None) -> None:
         """Add several pages to the Graphical User Interface.
 
         Arguments:
@@ -1322,8 +1331,9 @@ class Gui:
         to = to or Gui.__root_page_name
         if to not in self._config.routes:
             warnings.warn(f'Cannot navigate to "{to if to != Gui.__root_page_name else "/"}": unknown page.')
-            return
+            return False
         self.__send_ws_navigate(to)
+        return True
 
     def __init_route(self):
         self.__set_client_id_in_context()
@@ -1360,8 +1370,12 @@ class Gui:
         if hasattr(self, "on_navigate") and callable(self.on_navigate):
             try:
                 nav_page = self.on_navigate(self.__get_state(), page_name)
-                if not isinstance(nav_page, str):
-                    warnings.warn(f"on_navigate() returned a invalid page name '{nav_page}'.")
+                if nav_page != page_name:
+                    if isinstance(nav_page, str):
+                        if self._navigate(nav_page):
+                            return ("Root page cannot be re-routed by on_navigate().", 302)
+                    else:
+                        warnings.warn(f"on_navigate() returned an invalid page name '{nav_page}'.")
                     nav_page = page_name
             except Exception as e:
                 if not self._call_on_exception("on_navigate", e):
@@ -1472,6 +1486,128 @@ class Gui:
                     ]
         return config
 
+    def __init_server(self):
+        app_config = self._config.config
+        # Init server if there is no server
+        if not hasattr(self, "_server"):
+            self._server = _Server(
+                self,
+                path_mapping=self._path_mapping,
+                flask=self._flask,
+                css_file=self._css_file,
+                async_mode=app_config["async_mode"],
+            )
+
+        # Stop and reinitialize the server if it is still running as a thread
+        if (_is_in_notebook() or app_config["run_in_thread"]) and hasattr(self._server, "_thread"):
+            self.stop()
+            self._flask_blueprint = []
+            self._server = _Server(
+                self,
+                path_mapping=self._path_mapping,
+                flask=self._flask,
+                css_file=self._css_file,
+                async_mode=app_config["async_mode"],
+            )
+            self._bindings()._new_scopes()
+
+    def __init_ngrok(self):
+        app_config = self._config.config
+        if app_config["run_server"] and app_config["ngrok_token"]:  # pragma: no cover
+            if not util.find_spec("pyngrok"):
+                raise RuntimeError("Cannot use ngrok as pyngrok package is not installed.")
+            ngrok.set_auth_token(app_config["ngrok_token"])
+            http_tunnel = ngrok.connect(app_config["port"], "http")
+            _TaipyLogger._get_logger().info(f" * NGROK Public Url: {http_tunnel.public_url}")
+
+    def __bind_default_function(self):
+        with self.get_flask_app().app_context():
+            self.__var_dir.process_imported_var()
+            # bind on_* function if available
+            self.__bind_local_func("on_init")
+            self.__bind_local_func("on_change")
+            self.__bind_local_func("on_action")
+            self.__bind_local_func("on_navigate")
+            self.__bind_local_func("on_exception")
+            self.__bind_local_func("on_status")
+
+    def __register_blueprint(self):
+        # add en empty main page if it is not defined
+        if Gui.__root_page_name not in self._config.routes:
+            new_page = _Page()
+            new_page._route = Gui.__root_page_name
+            new_page._renderer = _EmptyPage()
+            self._config.pages.append(new_page)
+            self._config.routes.append(Gui.__root_page_name)
+
+        pages_bp = Blueprint("taipy_pages", __name__)
+        self._flask_blueprint.append(pages_bp)
+
+        # server URL Rule for taipy images
+        images_bp = Blueprint("taipy_images", __name__)
+        images_bp.add_url_rule(f"{Gui.__CONTENT_ROOT}<path:path>", view_func=self.__serve_content)
+        self._flask_blueprint.append(images_bp)
+
+        # server URL for uploaded files
+        upload_bp = Blueprint("taipy_upload", __name__)
+        upload_bp.add_url_rule(Gui.__UPLOAD_URL, view_func=self.__upload_files, methods=["POST"])
+        self._flask_blueprint.append(upload_bp)
+
+        # server URL for extension resources
+        extension_bp = Blueprint("taipy_extensions", __name__)
+        extension_bp.add_url_rule(f"{Gui._EXTENSION_ROOT}<path:path>", view_func=self.__serve_extension)
+        scripts = [
+            s if bool(urlparse(s).netloc) else f"{Gui._EXTENSION_ROOT}{name}/{s}"
+            for name, libs in Gui.__extensions.items()
+            for lib in libs
+            for s in (lib.get_scripts() or [])
+        ]
+        styles = [
+            s if bool(urlparse(s).netloc) else f"{Gui._EXTENSION_ROOT}{name}/{s}"
+            for name, libs in Gui.__extensions.items()
+            for lib in libs
+            for s in (lib.get_styles() or [])
+        ]
+        self._flask_blueprint.append(extension_bp)
+
+        _conf_webapp_path = (
+            pathlib.Path(self._get_config("webapp_path", None)) if self._get_config("webapp_path", None) else None
+        )
+        _webapp_path = str((pathlib.Path(__file__).parent / "webapp").resolve())
+        if _conf_webapp_path:
+            if _conf_webapp_path.is_dir():
+                _webapp_path = str(_conf_webapp_path.resolve())
+                warnings.warn(f"Now using webapp_path: '{_conf_webapp_path}'.")
+            else:
+                warnings.warn(
+                    f"webapp_path: '{_conf_webapp_path}' is not a valid directory path. Falling back to '{_webapp_path}'."
+                )
+
+        self._flask_blueprint.append(
+            self._server._get_default_blueprint(
+                static_folder=_webapp_path,
+                template_folder=_webapp_path,
+                title=self._get_config("title", "Taipy App"),
+                favicon=self._get_config("favicon", "/favicon.png"),
+                root_margin=self._get_config("margin", None),
+                scripts=scripts,
+                styles=styles,
+                version=self.__get_version(),
+                client_config=self.__get_client_config(),
+                watermark=self._get_config("watermark", None),
+            )
+        )
+
+        # Run parse markdown to force variables binding at runtime
+        pages_bp.add_url_rule("/taipy-jsx/<path:page_name>", view_func=self.__render_page)
+
+        # server URL Rule for flask rendered react-router
+        pages_bp.add_url_rule("/taipy-init", view_func=self.__init_route)
+
+        # Register Flask Blueprint if available
+        for bp in self._flask_blueprint:
+            self._server.get_flask().register_blueprint(bp)
+
     def run(
         self,
         run_server: bool = True,
@@ -1538,49 +1674,21 @@ class Gui:
         if not hasattr(self, "_root_dir"):
             self._root_dir = run_root_dir
 
+        kwargs = {
+            **kwargs,
+            "run_server": run_server,
+            "run_in_thread": run_in_thread,
+            "async_mode": async_mode,
+        }
+
         # Load application config from multiple sources (env files, kwargs, command line)
         self._config._build_config(run_root_dir, self.__env_filename, kwargs)
 
-        if app_config["debug"] and async_mode != "threading":
-            async_mode = "threading"
-            warnings.warn(
-                "'async_mode' parameter is overridden to 'threading'. Using Flask Development Server in Debug mode."
-            )
+        self._config.resolve()
 
-        # Init server if there is no server
-        if not hasattr(self, "_server"):
-            self._server = _Server(
-                self,
-                path_mapping=self._path_mapping,
-                flask=self._flask,
-                css_file=self._css_file,
-                async_mode=async_mode,
-            )
+        self.__init_server()
 
-        # Stop and reinitialize the server if it is still running as a thread
-        if (_is_in_notebook() or run_in_thread) and hasattr(self._server, "_thread"):
-            self._server.stop_thread()
-            self._flask_blueprint = []
-            self._server = _Server(
-                self,
-                path_mapping=self._path_mapping,
-                flask=self._flask,
-                css_file=self._css_file,
-                async_mode=async_mode,
-            )
-            self._bindings()._new_scopes()
-
-        # Special config for notebook runtime
-        if _is_in_notebook() or run_in_thread:
-            self._config.config["single_client"] = True
-
-        if run_server and app_config["ngrok_token"]:  # pragma: no cover
-            if not util.find_spec("pyngrok"):
-                raise RuntimeError("Cannot use ngrok as pyngrok package is not installed.")
-            ngrok.set_auth_token(app_config["ngrok_token"])
-            http_tunnel = ngrok.connect(app_config["port"], "http")
-            app_config["use_reloader"] = False
-            print(f" * NGROK Public Url: {http_tunnel.public_url}")
+        self.__init_ngrok()
 
         locals_bind = _filter_locals(self.__frame.f_locals)
 
@@ -1595,84 +1703,14 @@ class Gui:
             # to allow gui.state.x in notebook mode
             self.state = self.__state
 
-        with self.get_flask_app().app_context():
-            self.__var_dir.process_imported_var()
-            # bind on_* function if available
-            self.__bind_local_func("on_init")
-            self.__bind_local_func("on_change")
-            self.__bind_local_func("on_action")
-            self.__bind_local_func("on_navigate")
-            self.__bind_local_func("on_exception")
-            self.__bind_local_func("on_status")
+        self.__bind_default_function()
 
         # base global ctx is TaipyHolder classes + script modules and callables
         glob_ctx = {t.__name__: t for t in _TaipyBase.__subclasses__()}
         glob_ctx.update({k: v for k, v in locals_bind.items() if inspect.ismodule(v) or callable(v)})
         self.__evaluator = _Evaluator(glob_ctx)
 
-        # add en empty main page if it is not defined
-        if Gui.__root_page_name not in self._config.routes:
-            new_page = _Page()
-            new_page._route = Gui.__root_page_name
-            new_page._renderer = _EmptyPage()
-            self._config.pages.append(new_page)
-            self._config.routes.append(Gui.__root_page_name)
-
-        pages_bp = Blueprint("taipy_pages", __name__)
-        self._flask_blueprint.append(pages_bp)
-
-        # server URL Rule for taipy images
-        images_bp = Blueprint("taipy_images", __name__)
-        images_bp.add_url_rule(f"{Gui.__CONTENT_ROOT}<path:path>", view_func=self.__serve_content)
-        self._flask_blueprint.append(images_bp)
-
-        # server URL for uploaded files
-        upload_bp = Blueprint("taipy_upload", __name__)
-        upload_bp.add_url_rule(Gui.__UPLOAD_URL, view_func=self.__upload_files, methods=["POST"])
-        self._flask_blueprint.append(upload_bp)
-
-        # server URL for extension resources
-        extension_bp = Blueprint("taipy_extensions", __name__)
-        extension_bp.add_url_rule(f"{Gui._EXTENSION_ROOT}<path:path>", view_func=self.__serve_extension)
-        scripts = [
-            f"{Gui._EXTENSION_ROOT}{name}/{s}"
-            for name, libs in Gui.__extensions.items()
-            for lib in libs
-            for s in (lib.get_scripts() or [])
-        ]
-        styles = [
-            f"{Gui._EXTENSION_ROOT}{name}/{s}"
-            for name, libs in Gui.__extensions.items()
-            for lib in libs
-            for s in (lib.get_styles() or [])
-        ]
-        self._flask_blueprint.append(extension_bp)
-
-        _absolute_path = str(pathlib.Path(__file__).parent.resolve())
-        self._flask_blueprint.append(
-            self._server._get_default_blueprint(
-                static_folder=f"{_absolute_path}{os.path.sep}webapp",
-                template_folder=f"{_absolute_path}{os.path.sep}webapp",
-                title=self._get_config("title", "Taipy App"),
-                favicon=self._get_config("favicon", "/favicon.png"),
-                root_margin=self._get_config("margin", None),
-                scripts=scripts,
-                styles=styles,
-                version=self.__get_version(),
-                client_config=self.__get_client_config(),
-                watermark=self._get_config("watermark", None),
-            )
-        )
-
-        # Run parse markdown to force variables binding at runtime
-        pages_bp.add_url_rule("/taipy-jsx/<path:page_name>", view_func=self.__render_page)
-
-        # server URL Rule for flask rendered react-router
-        pages_bp.add_url_rule("/taipy-init", view_func=self.__init_route)
-
-        # Register Flask Blueprint if available
-        for bp in self._flask_blueprint:
-            self._server.get_flask().register_blueprint(bp)
+        self.__register_blueprint()
 
         # Register data accessor communicaiton data format (JSON, Apache Arrow)
         self._accessors._set_data_format(_DataFormat.APACHE_ARROW if app_config["use_arrow"] else _DataFormat.JSON)
@@ -1684,13 +1722,14 @@ class Gui:
         if not run_server:
             return self.get_flask_app()
 
-        return self._server.runWithWS(
+        return self._server.run(
             host=app_config["host"],
             port=app_config["port"],
             debug=app_config["debug"],
             use_reloader=app_config["use_reloader"],
             flask_log=app_config["flask_log"],
-            run_in_thread=run_in_thread,
+            run_in_thread=app_config["run_in_thread"],
+            allow_unsafe_werkzeug=app_config["allow_unsafe_werkzeug"],
         )
 
     def stop(self):
@@ -1702,6 +1741,6 @@ class Gui:
         `(Gui.)run^` method was set to True, or you are running in an IPython notebook
         context.
         """
-        if hasattr(self, "_server") and hasattr(self._server, "_thread"):
+        if hasattr(self, "_server") and hasattr(self._server, "_thread") and self._server._is_running:
             self._server.stop_thread()
-            print("Gui server has been stopped.")
+            _TaipyLogger._get_logger().info("Gui server has been stopped.")
