@@ -1,4 +1,4 @@
-# Copyright 2022 Avaiga Private Limited
+# Copyright 2023 Avaiga Private Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
@@ -17,6 +17,7 @@ import typing as t
 import warnings
 import xml.etree.ElementTree as etree
 from datetime import date, datetime, time
+from enum import Enum
 from inspect import isclass
 
 from ..partial import Partial
@@ -47,6 +48,40 @@ if t.TYPE_CHECKING:
     from ..gui import Gui
 
 
+class _Chart_iprops(Enum):
+    x = 0
+    y = 1
+    z = 2
+    label = 3
+    text = 4
+    mode = 5
+    type = 6
+    color = 7
+    xaxis = 8
+    yaxis = 9
+    selected_color = 10
+    marker = 11
+    selected_marker = 12
+    orientation = 13
+    _name = 14
+    line = 15
+    text_anchor = 16
+    options = 17
+    lon = 18
+    lat = 19
+    base = 20
+    r = 21
+    theta = 22
+    close = 23
+    open = 24
+    high = 25
+    low = 26
+    locations = 27
+    values = 28
+    labels = 29
+    decimator = 30
+
+
 class _Builder:
     """
     Constructs an XML node that can be rendered as a React node.
@@ -56,16 +91,36 @@ class _Builder:
 
     __keys: t.Dict[str, int] = {}
 
-    __ONE_COLUMN_CHART = ["pie"]
-
     __BLOCK_CONTROLS = ["dialog", "expandable", "pane", "part"]
 
+    __CHART_AXIS: t.Dict[str, t.Iterable[_Chart_iprops]] = {
+        "scattermapbox": (_Chart_iprops.lon, _Chart_iprops.lat),
+        "scattergeo": (_Chart_iprops.lon, _Chart_iprops.lat),
+        "densitymapbox": (_Chart_iprops.lon, _Chart_iprops.lat, _Chart_iprops.z),
+        "scatterpolar": (_Chart_iprops.r, _Chart_iprops.theta),
+        "scatterpolargl": (_Chart_iprops.r, _Chart_iprops.theta),
+        "candlestick": (
+            _Chart_iprops.x,
+            _Chart_iprops.close,
+            _Chart_iprops.open,
+            _Chart_iprops.high,
+            _Chart_iprops.low,
+        ),
+        "bar": (_Chart_iprops.x, _Chart_iprops.y, _Chart_iprops.base),
+        "pie": (_Chart_iprops.values, _Chart_iprops.labels),
+        "choropleth": (_Chart_iprops.locations, _Chart_iprops.z),
+        "funnelarea": (_Chart_iprops.values,),
+    }
+    __CHART_DEFAULT_AXIS: t.Iterable[_Chart_iprops] = (_Chart_iprops.x, _Chart_iprops.y, _Chart_iprops.z)
+    __CHART_MARKER_TO_COLS: t.Iterable[str] = ("color", "size", "symbol", "opacity")
+    __CHART_NO_INDEX: t.Iterable[str] = ("pie", "histogram", "heatmap", "funnelarea")
+
     def __init__(
         self,
         gui: "Gui",
         control_type: str,
         element_name: str,
-        attributes: t.Union[t.Dict[str, t.Any], None],
+        attributes: t.Optional[t.Dict[str, t.Any]],
         hash_names: t.Dict[str, str] = {},
         default_value="<Empty>",
         lib_name: str = "taipy",
@@ -208,7 +263,7 @@ class _Builder:
         """
         return self.__set_react_attribute(_to_camel_case(name), value)
 
-    def set_dict_attribute(self, name: str):
+    def set_dict_attribute(self, name: str, default_value: t.Optional[t.Dict[str, t.Any]] = None):
         """
         TODO
         Defines a React attribute as a stringified json dict.
@@ -216,8 +271,12 @@ class _Builder:
 
         Arguments:
             name (str): The property name.
+            default value (dict): iused if no value is specified
         """
-        if dict_attr := self.__attributes.get(name):
+        dict_attr = self.__attributes.get(name)
+        if dict_attr is None:
+            dict_attr = default_value
+        if dict_attr is not None:
             if isinstance(dict_attr, str):
                 vals = [x.strip().split(":") for x in dict_attr.split(";")]
                 dict_attr = {val[0].strip(): val[1].strip() for val in vals if len(val) > 1}
@@ -254,10 +313,17 @@ class _Builder:
             if not optional:
                 warnings.warn(f"Property {name} is required for control {self.__control_type}")
             return self
-        try:
-            val = float(value)
-        except ValueError:
-            raise ValueError(f"Property {name} expects a number for control {self.__control_type}")
+        if isinstance(value, str):
+            try:
+                val = float(value)
+            except ValueError:
+                raise ValueError(f"Property {name} expects a number for control {self.__control_type}")
+        elif isinstance(value, numbers.Number):
+            val = value  # type: ignore
+        else:
+            raise ValueError(
+                f"Property {name} expects a number for control {self.__control_type}, received {type(value)}"
+            )
         return self.__set_react_attribute(_to_camel_case(name), val)
 
     def __set_string_attribute(
@@ -271,16 +337,23 @@ class _Builder:
         return self.set_attribute(_to_camel_case(name), str(strattr))
 
     def __set_dynamic_string_attribute(
-        self, name: str, default_value: t.Optional[str] = None, with_update: t.Optional[bool] = False
+        self,
+        name: str,
+        default_value: t.Optional[str] = None,
+        with_update: t.Optional[bool] = False,
+        dynamic_property_name: t.Optional[str] = None,
     ):
         hash_name = self.__hashes.get(name)
         str_val = self.__attributes.get(name, default_value)
         if str_val is not None:
-            self.set_attribute(_to_camel_case(f"default_{name}"), str(str_val))
+            self.set_attribute(
+                _to_camel_case(f"default_{name}" if dynamic_property_name is None else name), str(str_val)
+            )
         if hash_name:
+            prop_name = _to_camel_case(name if dynamic_property_name is None else dynamic_property_name)
             if with_update:
-                self.__update_vars.append(f"{name}={hash_name}")
-            self.__set_react_attribute(name, hash_name)
+                self.__update_vars.append(f"{prop_name}={hash_name}")
+            self.__set_react_attribute(prop_name, hash_name)
         return self
 
     def __set_function_attribute(
@@ -302,12 +375,6 @@ class _Builder:
                 warnings.warn(f" {self.__control_type}.{name}: {strattr} is not a function")
         return self.set_attribute(_to_camel_case(name), strattr) if strattr else self
 
-    def __set_decimator_attribute(self, attr_name: str):
-        strattr = self.__attributes.get(attr_name)
-        cls = self.__gui._get_user_instance(class_name=str(strattr), class_type=PropertyType.decimator.value)
-        if isinstance(cls, PropertyType.decimator.value):
-            self.__set_string_or_number_attribute(attr_name, strattr)
-
     def __set_string_or_number_attribute(self, name: str, default_value: t.Optional[t.Any] = None):
         attr = self.__attributes.get(name, default_value)
         if attr is None:
@@ -391,7 +458,7 @@ class _Builder:
     def __update_col_desc_from_indexed(self, columns: t.Dict[str, t.Any], name: str):
         col_value = self.get_name_indexed_property(name)
         for k, v in col_value.items():
-            if col_desc := next((x for x in columns.values() if x["dfid"] == k), None):
+            if col_desc := next((x for x in columns.values() if x.get("dfid") == k), None):
                 if col_desc.get(_to_camel_case(name)) is None:
                     col_desc[_to_camel_case(name)] = str(v)
             else:
@@ -414,27 +481,27 @@ class _Builder:
             filters = self.get_name_indexed_property("filter")
             for k, v in filters.items():
                 if _is_boolean_true(v):
-                    if col_desc := next((x for x in columns.values() if x["dfid"] == k), None):
+                    if col_desc := next((x for x in columns.values() if x.get("dfid") == k), None):
                         col_desc["filter"] = True
                     else:
                         warnings.warn(f"{self.__element_name} filter[{k}] is not in the list of displayed columns")
             editables = self.get_name_indexed_property("editable")
             for k, v in editables.items():
                 if _is_boolean(v):
-                    if col_desc := next((x for x in columns.values() if x["dfid"] == k), None):
+                    if col_desc := next((x for x in columns.values() if x.get("dfid") == k), None):
                         col_desc["notEditable"] = not _is_boolean_true(v)
                     else:
                         warnings.warn(f"{self.__element_name} editable[{k}] is not in the list of displayed columns")
             group_by = self.get_name_indexed_property("group_by")
             for k, v in group_by.items():
                 if _is_boolean_true(v):
-                    if col_desc := next((x for x in columns.values() if x["dfid"] == k), None):
+                    if col_desc := next((x for x in columns.values() if x.get("dfid") == k), None):
                         col_desc["groupBy"] = True
                     else:
                         warnings.warn(f"{self.__element_name} group_by[{k}] is not in the list of displayed columns")
             apply = self.get_name_indexed_property("apply")
             for k, v in apply.items():  # pragma: no cover
-                if col_desc := next((x for x in columns.values() if x["dfid"] == k), None):
+                if col_desc := next((x for x in columns.values() if x.get("dfid") == k), None):
                     if callable(v):
                         value = self.__hashes.get(f"apply[{k}]")
                     elif isinstance(v, str):
@@ -459,7 +526,7 @@ class _Builder:
                     self.set_attribute("lineStyle", value)
             styles = self.get_name_indexed_property("style")
             for k, v in styles.items():  # pragma: no cover
-                if col_desc := next((x for x in columns.values() if x["dfid"] == k), None):
+                if col_desc := next((x for x in columns.values() if x.get("dfid") == k), None):
                     if callable(v):
                         value = self.__hashes.get(f"style[{k}]")
                     elif isinstance(v, str):
@@ -472,145 +539,214 @@ class _Builder:
                         col_desc["style"] = value
                 else:
                     warnings.warn(f"{self.__element_name} style[{k}] is not in the list of displayed columns")
+            if tooltip := self.__attributes.get("tooltip"):
+                if callable(tooltip):
+                    value = self.__hashes.get("tooltip")
+                elif isinstance(tooltip, str):
+                    value = tooltip.strip()
+                else:
+                    value = None
+                if value in col_types.keys():
+                    warnings.warn(f"{self.__element_name} tooltip={value} cannot be a column's name")
+                elif value:
+                    self.set_attribute("tooltip", value)
+            tooltips = self.get_name_indexed_property("tooltip")
+            for k, v in tooltips.items():  # pragma: no cover
+                if col_desc := next((x for x in columns.values() if x.get("dfid") == k), None):
+                    if callable(v):
+                        value = self.__hashes.get(f"tooltip[{k}]")
+                    elif isinstance(v, str):
+                        value = v.strip()
+                    else:
+                        value = None
+                    if value in col_types.keys():
+                        warnings.warn(f"{self.__element_name} tooltip[{k}]={value} cannot be a column's name")
+                    elif value:
+                        col_desc["tooltip"] = value
+                else:
+                    warnings.warn(f"{self.__element_name} tooltip[{k}] is not in the list of displayed columns")
             self.__attributes["columns"] = columns
             self.__set_json_attribute("columns", columns)
         return self
 
-    def __check_dict(self, values: t.List[t.Any], indexes: t.Tuple[int], names: t.Tuple[str]) -> None:
-        for index in indexes:
-            if values[index] is not None and not isinstance(values[index], (dict, _MapDict)):
-                warnings.warn(f"{self.__element_name} {names[index]} should be a dict")
-                values[index] = None
+    def __check_dict(self, values: t.List[t.Any], properties: t.Tuple[_Chart_iprops]) -> None:
+        for prop in properties:
+            if values[prop.value] is not None and not isinstance(values[prop.value], (dict, _MapDict)):
+                warnings.warn(f"{self.__element_name} {prop.name} should be a dict")
+                values[prop.value] = None
 
     def _get_chart_config(self, default_type="scatter", default_mode="lines+markers"):  # noqa: C901
-        names = (
-            "x",  # 0
-            "y",
-            "z",  # 2
-            "label",
-            "text",  # 4
-            "mode",
-            "type",  # 6
-            "color",
-            "xaxis",  # 8
-            "yaxis",
-            "selected_color",  # 10
-            "marker",
-            "selected_marker",  # 12
-            "orientation",
-            "name",  # 14
-            "line",
-            "text_anchor",  # 16
-            "options",
-            "lon",  # 18
-            "lat",
-            "base",  # 20
-        )
+        names = tuple(e.name[1:] if e.name[0] == "_" else e.name for e in _Chart_iprops)
         trace = self.__get_multiple_indexed_attributes(names)
-        if not trace[0] and trace[18]:
-            trace[0] = trace[18]  # substitute Lon to x
-        if not trace[1] and trace[19]:
-            trace[1] = trace[19]  # substitute Lat to y
-        if not trace[5]:
-            # mode
-            trace[5] = default_mode
-        trace[6] = str(trace[6]).strip().lower() if trace[6] else default_type
-        if not trace[8]:
-            # xaxis
-            trace[8] = "x"
-        if not trace[9]:
-            # yaxis
-            trace[9] = "y"
-        self.__check_dict(trace, (11, 12, 17), names)
+        if not trace[_Chart_iprops.mode.value]:
+            trace[_Chart_iprops.mode.value] = default_mode
+        # type
+        if not trace[_Chart_iprops.type.value]:
+            trace[_Chart_iprops.type.value] = default_type
+        if not trace[_Chart_iprops.xaxis.value]:
+            trace[_Chart_iprops.xaxis.value] = "x"
+        if not trace[_Chart_iprops.yaxis.value]:
+            trace[_Chart_iprops.yaxis.value] = "y"
+        # Indexed properties: Check for arrays
+        for prop in _Chart_iprops:
+            values = trace[prop.value]
+            if isinstance(values, (list, tuple)) and len(values):
+                prop_name = prop.name[1:] if prop.name[0] == "_" else prop.name
+                for idx, val in enumerate(values):
+                    if idx == 0:
+                        trace[prop.value] = val
+                    if val is not None:
+                        indexed_prop = f"{prop_name}[{idx + 1}]"
+                        if self.__attributes.get(indexed_prop) is None:
+                            self.__attributes[indexed_prop] = val
+        # marker selected_marker options
+        self.__check_dict(trace, (_Chart_iprops.marker, _Chart_iprops.selected_marker, _Chart_iprops.options))
+        axis = []
         traces = []
         idx = 1
         indexed_trace = self.__get_multiple_indexed_attributes(names, idx)
         if len([x for x in indexed_trace if x]):
             while len([x for x in indexed_trace if x]):
-                if not indexed_trace[0] and indexed_trace[18]:
-                    indexed_trace[0] = indexed_trace[18]  # substitute Lon to x
-                if not indexed_trace[1] and indexed_trace[19]:
-                    indexed_trace[1] = indexed_trace[19]  # substitute Lat to y
-                self.__check_dict(indexed_trace, (11, 12, 17), names)
+                axis.append(
+                    _Builder.__CHART_AXIS.get(
+                        indexed_trace[_Chart_iprops.type.value] or trace[_Chart_iprops.type.value],
+                        _Builder.__CHART_DEFAULT_AXIS,
+                    )
+                )
+                # marker selected_marker options
+                self.__check_dict(
+                    indexed_trace, (_Chart_iprops.marker, _Chart_iprops.selected_marker, _Chart_iprops.options)
+                )
+                if trace[_Chart_iprops.decimator.value] and not indexed_trace[_Chart_iprops.decimator.value]:
+                    # copy the decimator only once
+                    indexed_trace[_Chart_iprops.decimator.value] = trace[_Chart_iprops.decimator.value]
+                    trace[_Chart_iprops.decimator.value] = None
                 traces.append([x or trace[i] for i, x in enumerate(indexed_trace)])
                 idx += 1
                 indexed_trace = self.__get_multiple_indexed_attributes(names, idx)
         else:
             traces.append(trace)
+            # axis names
+            axis.append(_Builder.__CHART_AXIS.get(trace[_Chart_iprops.type.value], _Builder.__CHART_DEFAULT_AXIS))
 
         # read column definitions
         data = self.__attributes.get("data")
         data_hash = self.__hashes.get("data", "")
         col_types = self.__gui._accessors._get_col_types(data_hash, _TaipyData(data, data_hash))
 
-        # add trace for non used indexed columns
-        max_idx = max(_get_idx_from_col(c) for c in col_types.keys())
-        traces.extend([x if i > 4 else None for i, x in enumerate(traces[0])] for _ in range(len(traces), max_idx + 1))
+        # list of data columns name indexes with label text
+        dt_idx = tuple(e.value for e in axis[0] + (_Chart_iprops.label, _Chart_iprops.text))
 
         # configure columns
         columns = set()
-        for trace in traces:
-            columns.update([t for t in trace[:5] if t])
-            if trace[20]:
-                columns.add(trace[20])
+        for j, trace in enumerate(traces):
+            dt_idx = tuple(
+                e.value for e in (axis[j] if j < len(axis) else axis[0]) + (_Chart_iprops.label, _Chart_iprops.text)
+            )
+            columns.update([trace[i] for i in dt_idx if trace[i]])
         # add optionnal column if any
-        markers = [t[11] or ({"color": t[7]} if t[7] else None) for t in traces]
+        markers = [
+            t[_Chart_iprops.marker.value]
+            or ({"color": t[_Chart_iprops.color.value]} if t[_Chart_iprops.color.value] else None)
+            for t in traces
+        ]
         opt_cols = set()
         for m in markers:
             if isinstance(m, (dict, _MapDict)):
-                color = m.get("color")
-                if isinstance(color, str) and color not in columns:
-                    opt_cols.add(color)
-                size = m.get("size")
-                if isinstance(size, str) and size not in columns:
-                    opt_cols.add(size)
+                for prop in _Builder.__CHART_MARKER_TO_COLS:
+                    val = m.get(prop)
+                    if isinstance(val, str) and val not in columns:
+                        opt_cols.add(val)
 
         # Validate the column names
         columns = _get_columns_dict(data, list(columns), col_types, opt_columns=opt_cols)
+
+        # Manage Decimator
+        decimators = []
+        for tr in traces:
+            if tr[_Chart_iprops.decimator.value]:
+                cls = self.__gui._get_user_instance(
+                    class_name=str(tr[_Chart_iprops.decimator.value]), class_type=PropertyType.decimator.value
+                )
+                if isinstance(cls, PropertyType.decimator.value):
+                    decimators.append(str(tr[_Chart_iprops.decimator.value]))
+                    continue
+            decimators.append(None)
+
         # set default columns if not defined
-        icols = [[c for c in [_get_col_from_indexed(c, i) for c in columns.keys()] if c] for i in range(len(traces))]
+        icols = [
+            [c2 for c2 in [_get_col_from_indexed(c1, i) for c1 in columns.keys()] if c2] for i in range(len(traces))
+        ]
 
         for i, tr in enumerate(traces):
-            if not tr[0] or tr[6] in _Builder.__ONE_COLUMN_CHART or not tr[1]:
-                traces[i] = tuple(
-                    v or (icols[i].pop(0) if j < 3 and j < len(icols[i]) else v) for j, v in enumerate(tr)
-                )
+            if i < len(axis):
+                used_cols = {tr[ax.value] for ax in axis[i] if tr[ax.value]}
+                unused_cols = [c for c in icols[i] if c not in used_cols]
+                if unused_cols and not any(tr[ax.value] for ax in axis[i]):
+                    traces[i] = tuple(
+                        v or (unused_cols.pop(0) if unused_cols and _Chart_iprops(j) in axis[i] else v)
+                        for j, v in enumerate(tr)
+                    )
 
         if columns is not None:
             self.__attributes["columns"] = columns
-            reverse_cols = {cd["dfid"]: c for c, cd in columns.items()}
+            reverse_cols = {str(cd.get("dfid")): c for c, cd in columns.items()}
+
+            # List used axis
+            used_axis = [
+                [e for e in (axis[j] if j < len(axis) else axis[0]) if tr[e.value]] for j, tr in enumerate(traces)
+            ]
 
             ret_dict = {
                 "columns": columns,
-                "labels": [reverse_cols.get(t[3], (t[3] or "")) for t in traces],
-                "texts": [reverse_cols.get(t[4], (t[4] or None)) for t in traces],
-                "modes": [t[5] for t in traces],
-                "types": [t[6] for t in traces],
-                "xaxis": [t[8] for t in traces],
-                "yaxis": [t[9] for t in traces],
+                "labels": [
+                    reverse_cols.get(tr[_Chart_iprops.label.value], (tr[_Chart_iprops.label.value] or ""))
+                    for tr in traces
+                ],
+                "texts": [
+                    reverse_cols.get(tr[_Chart_iprops.text.value], (tr[_Chart_iprops.text.value] or None))
+                    for tr in traces
+                ],
+                "modes": [tr[_Chart_iprops.mode.value] for tr in traces],
+                "types": [tr[_Chart_iprops.type.value] for tr in traces],
+                "xaxis": [tr[_Chart_iprops.xaxis.value] for tr in traces],
+                "yaxis": [tr[_Chart_iprops.yaxis.value] for tr in traces],
                 "markers": markers,
-                "selectedMarkers": [t[12] or ({"color": t[10]} if t[10] else None) for t in traces],
-                "traces": [[reverse_cols.get(c, c) for c in [t[0], t[1], t[2]]] for t in traces],
-                "orientations": [t[13] for t in traces],
-                "names": [t[14] for t in traces],
-                "lines": [t[15] if isinstance(t[15], dict) else {"dash": t[15]} for t in traces],
-                "textAnchors": [t[16] for t in traces],
-                "options": [t[17] for t in traces],
-                "bases": [reverse_cols.get(t[20], (t[20] or "")) for t in traces],
+                "selectedMarkers": [
+                    tr[_Chart_iprops.selected_marker.value]
+                    or (
+                        {"color": tr[_Chart_iprops.selected_color.value]}
+                        if tr[_Chart_iprops.selected_color.value]
+                        else None
+                    )
+                    for tr in traces
+                ],
+                "traces": [
+                    [reverse_cols.get(c, c) for c in [tr[e.value] for e in used_axis[j]]] for j, tr in enumerate(traces)
+                ],
+                "orientations": [tr[_Chart_iprops.orientation.value] for tr in traces],
+                "names": [tr[_Chart_iprops._name.value] for tr in traces],
+                "lines": [
+                    tr[_Chart_iprops.line.value]
+                    if isinstance(tr[_Chart_iprops.line.value], (dict, _MapDict))
+                    else {"dash": tr[_Chart_iprops.line.value]}
+                    if tr[_Chart_iprops.line.value]
+                    else None
+                    for tr in traces
+                ],
+                "textAnchors": [tr[_Chart_iprops.text_anchor.value] for tr in traces],
+                "options": [tr[_Chart_iprops.options.value] for tr in traces],
+                "axisNames": [[e.name for e in ax] for ax in used_axis],
+                "addIndex": [tr[_Chart_iprops.type.value] not in _Builder.__CHART_NO_INDEX for tr in traces],
             }
+            if len([d for d in decimators if d]):
+                ret_dict.update(decimators=decimators)
 
             self.__set_json_attribute("config", ret_dict)
             self._set_chart_selected(max=len(traces))
             self.__set_refresh_on_update()
         return self
 
-    def _set_chart_layout(self):
-        if layout := self.__attributes.get("layout"):
-            if isinstance(layout, (dict, _MapDict)):
-                self.__set_json_attribute("layout", layout)
-            else:
-                warnings.warn(f"Chart control: layout attribute should be a dict\n'{str(layout)}'")
-
     def _set_string_with_check(self, var_name: str, values: t.List[str], default_value: t.Optional[str] = None):
         value = self.__attributes.get(var_name, default_value)
         if value is not None:
@@ -679,11 +815,8 @@ class _Builder:
         return self
 
     def __set_class_names(self):
-        classes = [self.__lib_name + "-" + self.__control_type.replace("_", "-")]
-        if cl := self.__attributes.get("class_name"):
-            classes.append(str(cl))
-
-        return self.set_attribute("className", " ".join(classes))
+        self.set_attribute("libClassName", self.__lib_name + "-" + self.__control_type.replace("_", "-"))
+        return self.__set_dynamic_string_attribute("class_name", dynamic_property_name="dynamic_class_name")
 
     def _set_dataType(self):
         value = self.__attributes.get("value")
@@ -958,7 +1091,7 @@ class _Builder:
             elif var_type == PropertyType.string_or_number:
                 self.__set_string_or_number_attribute(attr[0], _get_tuple_val(attr, 2, None))
             elif var_type == PropertyType.dict:
-                self.set_dict_attribute(attr[0])
+                self.set_dict_attribute(attr[0], _get_tuple_val(attr, 2, None))
             elif var_type == PropertyType.dynamic_list:
                 self.__set_dynamic_string_list(attr[0], _get_tuple_val(attr, 2, None))
             elif var_type == PropertyType.boolean_or_list:
@@ -966,8 +1099,6 @@ class _Builder:
                     self.__set_dynamic_bool_attribute(attr[0], _get_tuple_val(attr, 2, False), True, update_main=False)
                 else:
                     self.__set_dynamic_string_list(attr[0], _get_tuple_val(attr, 2, None))
-            elif var_type == PropertyType.decimator:
-                self.__set_decimator_attribute(attr_name=attr[0])
             elif var_type == PropertyType.data:
                 self.__set_dynamic_property_without_default(attr[0], var_type)
             elif var_type == PropertyType.lov:
