@@ -1,4 +1,4 @@
-# Copyright 2022 Avaiga Private Limited
+# Copyright 2023 Avaiga Private Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
@@ -95,10 +95,12 @@ class _Factory:
                 ("plot_config", PropertyType.dict),
                 ("on_range_change", PropertyType.function),
                 ("active", PropertyType.dynamic_boolean, True),
-                ("decimator", PropertyType.decimator),
                 ("render", PropertyType.dynamic_boolean, True),
                 ("hover_text", PropertyType.dynamic_string),
                 ("on_change", PropertyType.function),
+                ("template", PropertyType.dict),
+                ("template[dark]", PropertyType.dict, gui._get_config("chart_dark_template", None)),
+                ("template[light]", PropertyType.dict),
             ]
         )
         ._get_chart_config("scatter", "lines+markers")
@@ -431,7 +433,7 @@ class _Factory:
         ._get_dataframe_attributes()
         .set_attributes(
             [
-                ("page_size", PropertyType.react, 100),
+                ("page_size", PropertyType.number, "100"),
                 ("allow_all_rows", PropertyType.boolean),
                 ("show_all", PropertyType.boolean),
                 ("auto_loading", PropertyType.boolean),
@@ -443,6 +445,7 @@ class _Factory:
                 ("on_edit", PropertyType.function),
                 ("on_delete", PropertyType.function),
                 ("on_add", PropertyType.function),
+                ("on_action", PropertyType.function),
                 ("nan_value",),
                 ("filter", PropertyType.boolean),
                 ("hover_text", PropertyType.dynamic_string),
@@ -461,11 +464,7 @@ class _Factory:
         .set_value_and_default(with_update=False)
         ._set_dataType()
         .set_attributes(
-            [
-                ("format",),
-                ("id",),
-                ("hover_text", PropertyType.dynamic_string),
-            ]
+            [("format",), ("id",), ("hover_text", PropertyType.dynamic_string), ("raw", PropertyType.boolean, False)]
         ),
         "toggle": lambda gui, control_type, attrs: _Builder(
             gui=gui, control_type=control_type, element_name="Toggle", attributes=attrs, default_value=None
