@@ -1,4 +1,4 @@
-# Copyright 2022 Avaiga Private Limited
+# Copyright 2023 Avaiga Private Limited
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
@@ -11,6 +11,14 @@
 
 import re
 
+import pandas as pd
+
 
 def _get_data_type(value):
-    return re.match(r"^<class '(.*?)'>", str(type(value))).group(1)
+    if pd.api.types.is_bool_dtype(value):
+        return "bool"
+    elif pd.api.types.is_integer_dtype(value):
+        return "int"
+    elif pd.api.types.is_float_dtype(value):
+        return "float"
+    return re.match(r"^<class '(.*\.)?(.*?)(\d\d)?'>", str(type(value))).group(2)
